@@ -36,7 +36,7 @@ import h5py
 import numpy as np
 from confapp import conf
 from joblib import Parallel, delayed
-from tqdm import tqdm
+from rich.progress import track
 
 from idtrackerai.blob import Blob
 from idtrackerai.utils.py_utils import interpolate_nans
@@ -80,9 +80,12 @@ class ListOfBlobs(object):
         --------
         :meth:`blob.Blob.overlaps_with`
         """
-        self.disconnect()
-        for frame_i in tqdm(
-            range(1, self.number_of_frames), desc="Connecting blobs "
+
+        assert not self.blobs_are_connected
+        # self.disconnect()
+
+        for frame_i in track(
+            range(1, self.number_of_frames), description="Connecting blobs "
         ):
             for (blob_0, blob_1) in itertools.product(
                 self.blobs_in_video[frame_i - 1], self.blobs_in_video[frame_i]
@@ -97,6 +100,7 @@ class ListOfBlobs(object):
             del blob.pixels_set
         self.blobs_are_connected = True
 
+    # This function is no longer used
     def disconnect(self):
         """Reinitialise the previous and next attributes of each blob.
 
@@ -110,20 +114,6 @@ class ListOfBlobs(object):
                 blob.next, blob.previous = [], []
         self.blobs_are_connected = False
 
-    # TODO: Check if used. Otherwise delete
-    # def connect(self):
-    #     """Connects blobs in subsequent frames by computing their overlapping"""
-    #     logger.info("Connecting list of blob objects")
-    #     self.compute_overlapping_between_subsequent_frames()
-
-    # TODO: call compute_overlapping_between_subsequent_frames instead
-    # def reconnect(self):
-    #     """Connects blobs in subsequent frames by computing their overlapping
-    #     and sets blobs_are_connected to True
-    #     """
-    #     logger.info("re-Connecting list of blob objects")
-    #     self.compute_overlapping_between_subsequent_frames()
-
     def save(self, path_to_save=None):
         """Saves instance of the class
 
@@ -132,8 +122,12 @@ class ListOfBlobs(object):
         path_to_save : str, optional
             Path where to save the object, by default None
         """
-        self.disconnect()
-        logger.info("saving blobs list at %s" % path_to_save)
+        if self.blobs_are_connected:
+            logger.info("Partially disconnecting blobs for saving ListOfBlobs")
+            for blobs_in_frame in self.blobs_in_video:
+                for blob in blobs_in_frame:
+                    blob.next = []
+        logger.info(f"Saving ListOfBlobs at {path_to_save}")
         np.save(path_to_save, self)
 
     @staticmethod
@@ -150,11 +144,17 @@ class ListOfBlobs(object):
         An instance of :class:`ListOfBlobs`.
 
         """
-        logger.info("loading blobs list from %s" % path_to_load_blob_list_file)
+        logger.info(f"Loading ListOfBlobs from {path_to_load_blob_list_file}")
         list_of_blobs = np.load(
             path_to_load_blob_list_file, allow_pickle=True
         ).item()
-        list_of_blobs.blobs_are_connected = False
+
+        if list_of_blobs.blobs_are_connected:
+            logger.info("Reconnecting blobs")
+            for blobs_in_frame in list_of_blobs.blobs_in_video:
+                for blob in blobs_in_frame:
+                    for prev_blob in blob.previous:
+                        prev_blob.next.append(blob)
         return list_of_blobs
 
     # TODO: This is part of fragmentation it should be somewhere else.
@@ -172,8 +172,8 @@ class ListOfBlobs(object):
         counter = 0
         possible_blob_indices = range(number_of_animals)
 
-        for blobs_in_frame in tqdm(
-            self.blobs_in_video, desc="assigning fragment identifier"
+        for blobs_in_frame in track(
+            self.blobs_in_video, description="Assigning fragment identifier"
         ):
             used_blob_indices = [
                 blob.blob_index
@@ -267,12 +267,12 @@ class ListOfBlobs(object):
     # TODO: the term identification_image should be changed.
     def set_images_for_identification(
         self,
-        episodes_start_end,
+        episodes,
         identification_images_file_paths,
         identification_image_size,
         number_of_animals,
         number_of_frames,
-        video_path,
+        video_paths,
         height,
         width,
     ):
@@ -302,32 +302,39 @@ class ListOfBlobs(object):
         width : int
             Width of a video frame considering the resolution reduction factor.
         """
-        Output = Parallel(n_jobs=conf.NUMBER_OF_JOBS_FOR_SETTING_ID_IMAGES)(
+        blobs_in_episodes = Parallel(
+            n_jobs=conf.NUMBER_OF_JOBS_FOR_SETTING_ID_IMAGES
+        )(
             delayed(self._set_identification_images_per_episode)(
                 identification_image_size,
                 number_of_animals,
                 number_of_frames,
-                video_path,
+                video_paths[video_path_index],
                 height,
                 width,
                 file,
-                self.blobs_in_video[start:end],
+                self.blobs_in_video[global_start:global_end],
             )
-            for file, (start, end) in tqdm(
+            for file, (
+                local_start,
+                local_end,
+                video_path_index,
+                global_start,
+                global_end,
+            ) in track(
                 list(
                     zip(
                         identification_images_file_paths,
-                        episodes_start_end,
+                        episodes,
                     )
                 ),
-                desc="Setting images for identification",
+                description="Setting images for identification",
             )
         )
-        self.blobs_in_video = [
-            blobs_in_frame
-            for blobs_in_episode in Output
-            for blobs_in_frame in blobs_in_episode
-        ]
+
+        for blobs_in_episode, episode_info in zip(blobs_in_episodes, episodes):
+            global_start, global_end = episode_info[-2:]
+            self.blobs_in_video[global_start:global_end] = blobs_in_episode
 
     @staticmethod
     def _set_identification_images_per_episode(
@@ -421,7 +428,9 @@ class ListOfBlobs(object):
                     fillvalue=np.nan,
                 )
 
-        for blobs_in_frame in tqdm(self.blobs_in_video, desc="Updating hdf5"):
+        for blobs_in_frame in track(
+            self.blobs_in_video, description="Updating hdf5"
+        ):
             for blob in blobs_in_frame:
                 episode = video.in_which_episode(blob.frame_number)
                 image = blob.identification_image_index
@@ -458,9 +467,9 @@ class ListOfBlobs(object):
             "accumulation_step",
         ]
 
-        for blobs_in_frame in tqdm(
+        for blobs_in_frame in track(
             self.blobs_in_video,
-            desc="updating list of blobs from list of fragments",
+            description="updating list of blobs from list of fragments",
         ):
             for blob in blobs_in_frame:
                 fragment = fragments[
