@@ -33,7 +33,7 @@ import os
 
 import cv2
 import numpy as np
-from tqdm import tqdm
+from rich.progress import track
 
 from idtrackerai.utils.py_utils import get_spaced_colors_util
 
@@ -127,7 +127,7 @@ def generate_trajectories_video(
 ):
 
     video_name = (
-        os.path.split(video_object.video_path)[-1].split(".")[0]
+        os.path.split(video_object.video_paths[0])[-1].split(".")[0]
         + "_tracked.avi"
     )
     colors = get_spaced_colors_util(
@@ -143,16 +143,16 @@ def generate_trajectories_video(
     )
 
     if len(video_object.video_paths) == 1:
-        cap = cv2.VideoCapture(video_object.video_path)
+        cap = cv2.VideoCapture(video_object.video_paths[0])
     else:
         cap = None
 
     if ending_frame is None:
         ending_frame = len(trajectories)
 
-    for frame_number in tqdm(
+    for frame_number in track(
         range(starting_frame, ending_frame),
-        desc="Generating video with trajectories...",
+        description="Generating video with trajectories...",
     ):
         frame = apply_func_on_frame(
             video_object,
