@@ -42,7 +42,6 @@ from confapp import conf
 from idtrackerai.tracker.accumulation_manager import AccumulationManager
 from idtrackerai.tracker.accumulator import perform_one_accumulation_step
 from idtrackerai.tracker.assigner import assign_remaining_fragments
-from idtrackerai.list_of_blobs import ListOfBlobs
 from idtrackerai.list_of_global_fragments import (
     ListOfGlobalFragments,
     create_list_of_global_fragments,
@@ -460,7 +459,7 @@ class TrackerAPI(object):
             plot_flag=False,
             return_store_objects=False,
             layers_to_optimize=conf.LAYERS_TO_OPTIMISE_ACCUMULATION,
-            video_path=self.video.video_path,
+            video_paths=self.video.video_paths,
         )
         # Save network params
         self.accumulation_network_params.save()
@@ -824,7 +823,7 @@ class TrackerAPI(object):
             plot_flag=False,
             return_store_objects=False,
             layers_to_optimize=conf.LAYERS_TO_OPTIMISE_ACCUMULATION,
-            video_path=self.video.video_path,
+            video_paths=self.video.video_paths,
         )
         self.video._pretraining_network_params = self.pretrain_network_params
 
@@ -1089,7 +1088,7 @@ class TrackerAPI(object):
             create_trajectories = self.create_trajectories
 
         self.video.individual_fragments_stats = (
-            self.list_of_fragments.get_stats(self.list_of_global_fragments)
+            self.list_of_fragments.get_stats()
         )
         self.video.compute_estimated_accuracy(self.list_of_fragments.fragments)
         self.list_of_fragments.save_light_list(self.video._accumulation_folder)
